@@ -20,6 +20,7 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/aws/amazon-ecs-init/ecs-init/backoff"
@@ -78,6 +79,11 @@ const (
 	// For more information on setns, please read this manpage:
 	// http://man7.org/linux/man-pages/man2/setns.2.html
 	CapSysAdmin = "SYS_ADMIN"
+	// CapChown to start agent with CAP_CHOWN capability
+	// This is needed for the ECS Agent to invoke the chown call when
+	// configuring the files for configuration or administration.
+	// http://man7.org/linux/man-pages/man2/chown.2.html
+	CapChown = "CAP_CHOWN"
 	// DefaultCgroupMountpoint is the default mount point for the cgroup subsystem
 	DefaultCgroupMountpoint = "/sys/fs/cgroup"
 	// pluginSocketFilesDir specifies the location of UNIX domain socket files of
@@ -116,9 +122,7 @@ const (
 	containerResourcesRootDir = "/managed-agents"
 
 	execCapabilityName     = "execute-command"
-	execBinRelativePath    = "bin"
 	execConfigRelativePath = "config"
-	execCertsRelativePath  = "certs"
 
 	execAgentLogRelativePath = "/exec"
 )
@@ -129,33 +133,42 @@ var pluginDirs = []string{
 	pluginSpecFilesUsrDir,
 }
 
-var isPathValid = defaultIsPathValid
+var (
+	dockerOnce      sync.Once
+	dockerClient    *client
+	dockerClientErr error
+	isPathValid     = defaultIsPathValid
+)
 
-// Client enables business logic for running the Agent inside Docker
-type Client struct {
+// client enables business logic for running the Agent inside Docker
+type client struct {
 	docker dockerclient
 	fs     fileSystem
 }
 
-// NewClient reutrns a new Client
-func NewClient() (*Client, error) {
-	// Create a backoff for pinging the docker socker. This should result in 17-19
-	// seconds of delay in the worst-case between different actions that depend on
-	// docker
-	pingBackoff := backoff.NewBackoff(minBackoffDuration, maxBackoffDuration, backoffJitterMultiple,
-		backoffMultiple, maxRetries)
-	client, err := newDockerClient(godockerClientFactory{}, pingBackoff)
-	if err != nil {
-		return nil, err
-	}
-	return &Client{
-		docker: client,
-		fs:     standardFS,
-	}, nil
+// Client returns the global docker client.
+func Client() (*client, error) {
+	dockerOnce.Do(func() {
+		// Create a backoff for pinging the docker socket. This should result in 17-19
+		// seconds of delay in the worst-case between different actions that depend on
+		// docker
+		pingBackoff := backoff.NewBackoff(minBackoffDuration, maxBackoffDuration, backoffJitterMultiple,
+			backoffMultiple, maxRetries)
+		cl, err := newDockerClient(godockerClientFactory{}, pingBackoff)
+		if err != nil {
+			dockerClientErr = err
+			return
+		}
+		dockerClient = &client{
+			docker: cl,
+			fs:     standardFS,
+		}
+	})
+	return dockerClient, dockerClientErr
 }
 
 // IsAgentImageLoaded returns true if the Agent image is loaded in Docker
-func (c *Client) IsAgentImageLoaded() (bool, error) {
+func (c *client) IsAgentImageLoaded() (bool, error) {
 	images, err := c.docker.ListImages(godocker.ListImagesOptions{
 		All: true,
 	})
@@ -173,13 +186,13 @@ func (c *Client) IsAgentImageLoaded() (bool, error) {
 }
 
 // LoadImage loads an io.Reader into Docker
-func (c *Client) LoadImage(image io.Reader) error {
+func (c *client) LoadImage(image io.Reader) error {
 	return c.docker.LoadImage(godocker.LoadImageOptions{InputStream: image})
 }
 
 // RemoveExistingAgentContainer remvoes any existing container named
 // "ecs-agent" or returns without error if none is found
-func (c *Client) RemoveExistingAgentContainer() error {
+func (c *client) RemoveExistingAgentContainer() error {
 	containerToRemove, err := c.findAgentContainer()
 	if err != nil {
 		return err
@@ -196,7 +209,7 @@ func (c *Client) RemoveExistingAgentContainer() error {
 	return err
 }
 
-func (c *Client) findAgentContainer() (string, error) {
+func (c *client) findAgentContainer() (string, error) {
 	// TODO pagination
 	containers, err := c.docker.ListContainers(godocker.ListContainersOptions{
 		All: true,
@@ -220,7 +233,7 @@ func (c *Client) findAgentContainer() (string, error) {
 }
 
 // StartAgent starts the Agent in Docker and returns the exit code from the container
-func (c *Client) StartAgent() (int, error) {
+func (c *client) StartAgent() (int, error) {
 	envVarsFromFiles := c.LoadEnvVars()
 
 	hostConfig := c.getHostConfig(envVarsFromFiles)
@@ -242,7 +255,7 @@ func (c *Client) StartAgent() (int, error) {
 
 // GetContainerLogTail will return the last logWindowSize lines of logs for
 // the Agent Container.
-func (c *Client) GetContainerLogTail(logWindowSize string) string {
+func (c *client) GetContainerLogTail(logWindowSize string) string {
 	containerToLog, _ := c.findAgentContainer()
 	if containerToLog == "" {
 		log.Info("No existing container to take logs from.")
@@ -265,7 +278,7 @@ func (c *Client) GetContainerLogTail(logWindowSize string) string {
 	return containerLogBuf.String()
 }
 
-func (c *Client) getContainerConfig(envVarsFromFiles map[string]string) *godocker.Config {
+func (c *client) getContainerConfig(envVarsFromFiles map[string]string) *godocker.Config {
 	// default environment variables
 	envVariables := map[string]string{
 		"ECS_LOGFILE":                           logDir + "/" + config.AgentLogFile,
@@ -326,7 +339,7 @@ func setLabels(cfg *godocker.Config, labelsStringRaw string) {
 	}
 }
 
-func (c *Client) LoadEnvVars() map[string]string {
+func (c *client) LoadEnvVars() map[string]string {
 	envVariables := make(map[string]string)
 	// merge in instance-specific environment variables
 	for envKey, envValue := range c.loadCustomInstanceEnvVars() {
@@ -349,16 +362,16 @@ func (c *Client) LoadEnvVars() map[string]string {
 }
 
 // loadUsrEnvVars gets user-supplied environment variables
-func (c *Client) loadUsrEnvVars() map[string]string {
+func (c *client) loadUsrEnvVars() map[string]string {
 	return c.getEnvVars(config.AgentConfigFile())
 }
 
 // loadCustomInstanceEnvVars gets custom config set in the instance by Amazon
-func (c *Client) loadCustomInstanceEnvVars() map[string]string {
+func (c *client) loadCustomInstanceEnvVars() map[string]string {
 	return c.getEnvVars(config.InstanceConfigFile())
 }
 
-func (c *Client) getEnvVars(filename string) map[string]string {
+func (c *client) getEnvVars(filename string) map[string]string {
 	envVariables := make(map[string]string)
 
 	file, err := c.fs.ReadFile(filename)
@@ -384,7 +397,7 @@ func generateLabelMap(jsonBlock string) (map[string]string, error) {
 	return out, err
 }
 
-func (c *Client) getHostConfig(envVarsFromFiles map[string]string) *godocker.HostConfig {
+func (c *client) getHostConfig(envVarsFromFiles map[string]string) *godocker.HostConfig {
 	dockerSocketBind := getDockerSocketBind(envVarsFromFiles)
 
 	binds := []string{
@@ -422,7 +435,7 @@ func (c *Client) getHostConfig(envVarsFromFiles map[string]string) *godocker.Hos
 	binds = append(binds, getDockerPluginDirBinds()...)
 
 	// only add bind mounts when the src file/directory exists on host; otherwise docker API create an empty directory on host
-	binds = append(binds, getCapabilityExecBinds()...)
+	binds = append(binds, getCapabilityBinds()...)
 
 	return createHostConfig(binds)
 }
@@ -458,31 +471,24 @@ func getDockerPluginDirBinds() []string {
 	return pluginBinds
 }
 
-func getCapabilityExecBinds() []string {
-	hostResourcesDir := filepath.Join(hostResourcesRootDir, execCapabilityName)
-	containerResourcesDir := filepath.Join(containerResourcesRootDir, execCapabilityName)
+func getCapabilityBinds() []string {
+	var binds = []string{}
 
-	var binds []string
-
-	// bind mount the entire /host/dependency/path/execute-command/bin folder
-	hostBinDir := filepath.Join(hostResourcesDir, execBinRelativePath)
-	if isPathValid(hostBinDir, true) {
+	// bind mount the entire /host/dependency/path/ folder
+	// as readonly to support all managed dependencies
+	if isPathValid(hostResourcesRootDir, true) {
 		binds = append(binds,
-			hostBinDir+":"+filepath.Join(containerResourcesDir, execBinRelativePath)+readOnly)
+			hostResourcesRootDir+":"+containerResourcesRootDir+readOnly)
 	}
 
 	// bind mount the entire /host/dependency/path/execute-command/config folder
 	// in read-write mode to allow ecs-agent to write config files to host file system
 	// (docker will) create the config folder if it does not exist
-	hostConfigDir := filepath.Join(hostResourcesDir, execConfigRelativePath)
-	binds = append(binds,
-		hostConfigDir+":"+filepath.Join(containerResourcesDir, execConfigRelativePath))
-
-	// bind mount the entire /host/dependency/path/execute-command/certs folder
-	hostCertsDir := filepath.Join(hostResourcesDir, execCertsRelativePath)
-	if isPathValid(hostCertsDir, true) {
+	hostConfigDir := filepath.Join(hostResourcesRootDir, execCapabilityName, execConfigRelativePath)
+	// Check that execute-command folder is present not config folder
+	if isPathValid(filepath.Dir(hostConfigDir), true) {
 		binds = append(binds,
-			hostCertsDir+":"+filepath.Join(containerResourcesDir, execCertsRelativePath)+readOnly)
+			hostConfigDir+":"+filepath.Join(containerResourcesRootDir, execCapabilityName, execConfigRelativePath))
 	}
 
 	return binds
@@ -518,7 +524,7 @@ func FilePatternMatchForGPU(pattern string) ([]string, error) {
 }
 
 // StopAgent stops the Agent in docker if one is running
-func (c *Client) StopAgent() error {
+func (c *client) StopAgent() error {
 	id, err := c.findAgentContainer()
 	if err != nil {
 		return err
