@@ -16,6 +16,7 @@ package iptables
 import (
 	"fmt"
 	"os"
+	"strings"
 	"testing"
 
 	"github.com/golang/mock/gomock"
@@ -40,9 +41,16 @@ var (
 		"!", "--ctstate", "RELATED,ESTABLISHED,DNAT",
 		"-j", "DROP",
 	}
+	offhostIntrospectionInterface                 = "ens5"
 	blockIntrospectionOffhostAccessInputRouteArgs = []string{
 		"-p", "tcp",
-		"-i", "eth0",
+		"-i", offhostIntrospectionInterface,
+		"--dport", agentIntrospectionServerPort,
+		"-j", "DROP",
+	}
+	blockIntrospectionOffhostAccessInterfaceInputRouteArgs = []string{
+		"-p", "tcp",
+		"-i", "sn0",
 		"--dport", agentIntrospectionServerPort,
 		"-j", "DROP",
 	}
@@ -53,9 +61,30 @@ var (
 		"-j", "REDIRECT",
 		"--to-ports", localhostCredentialsProxyPort,
 	}
+
+	testIPV4RouteInput = `Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT                                                       
+ens5	00000000	01201FAC	0003	0	0	0	00000000	0	0	0                                                                               
+ens5	FEA9FEA9	00000000	0005	0	0	0	FFFFFFFF	0	0	0                                                                               
+ens5	00201FAC	00000000	0001	0	0	0	00F0FFFF	0	0	0
+`
 )
 
+func overrideIPRouteInput(ipv4RouteInput string) func() {
+	originalv4 := getDefaultNetworkInterfaceIPv4
+
+	getDefaultNetworkInterfaceIPv4 = func() (string, error) {
+		return scanIPv4RoutesForDefaultInterface(strings.NewReader(ipv4RouteInput))
+	}
+
+	return func() {
+		getDefaultNetworkInterfaceIPv4 = originalv4
+		// in real environment we'll only set it once, for testing we unset it after executing relevant test cases
+		defaultOffhostIntrospectionInterface = ""
+	}
+}
+
 func TestNewNetfilterRouteFailsWhenExecutableNotFound(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -66,38 +95,73 @@ func TestNewNetfilterRouteFailsWhenExecutableNotFound(t *testing.T) {
 	assert.Error(t, err, "Expected error when executable's path lookup fails")
 }
 
-func TestCreate(t *testing.T) {
+func TestNewNetfilterRouteWithDefaultOffhostIntrospectionInterfaceFallback(t *testing.T) {
+	defer overrideIPRouteInput("")()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
-	mockCmd := NewMockCmd(ctrl)
-	// Mock a successful execution of the iptables command to create the
-	// route
 	mockExec := NewMockExec(ctrl)
-	gomock.InOrder(
-		mockExec.EXPECT().LookPath(iptablesExecutable).Return("", nil),
-		mockExec.EXPECT().Command(iptablesExecutable,
-			expectedArgs("nat", "-A", "PREROUTING", preroutingRouteArgs)).Return(mockCmd),
-		mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
-		mockExec.EXPECT().Command(iptablesExecutable,
-			expectedArgs("filter", "-I", "INPUT", localhostTrafficFilterInputRouteArgs)).Return(mockCmd),
-		mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
-		mockExec.EXPECT().Command(iptablesExecutable,
-			expectedArgs("filter", "-I", "INPUT", blockIntrospectionOffhostAccessInputRouteArgs)).Return(mockCmd),
-		mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
-		mockExec.EXPECT().Command(iptablesExecutable,
-			expectedArgs("nat", "-A", "OUTPUT", outputRouteArgs)).Return(mockCmd),
-		mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
-	)
+	mockExec.EXPECT().LookPath(iptablesExecutable).Return("", nil)
 
-	route, err := NewNetfilterRoute(mockExec)
-	require.NoError(t, err, "Error creating netfilter route object")
+	_, err := NewNetfilterRoute(mockExec)
+	assert.NoError(t, err)
+	assert.Equal(t, defaultOffhostIntrospectionInterface, fallbackOffhostIntrospectionInterface)
+}
+
+func TestCreate(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
+	testCases := []struct {
+		setOffhostInterface bool
+		inputRouteArgs      []string
+	}{
+		{
+			setOffhostInterface: false,
+			inputRouteArgs:      blockIntrospectionOffhostAccessInputRouteArgs,
+		},
+		{
+			setOffhostInterface: true,
+			inputRouteArgs:      blockIntrospectionOffhostAccessInterfaceInputRouteArgs,
+		},
+	}
+	for _, tc := range testCases {
+		if tc.setOffhostInterface {
+			os.Setenv(offhostIntrospectonAccessInterfaceEnv, "sn0")
+			defer os.Unsetenv(offhostIntrospectonAccessInterfaceEnv)
+		}
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
+
+		mockCmd := NewMockCmd(ctrl)
+		// Mock a successful execution of the iptables command to create the
+		// route
+		mockExec := NewMockExec(ctrl)
+		gomock.InOrder(
+			mockExec.EXPECT().LookPath(iptablesExecutable).Return("", nil),
+			mockExec.EXPECT().Command(iptablesExecutable,
+				expectedArgs("nat", "-A", "PREROUTING", preroutingRouteArgs)).Return(mockCmd),
+			mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
+			mockExec.EXPECT().Command(iptablesExecutable,
+				expectedArgs("filter", "-I", "INPUT", localhostTrafficFilterInputRouteArgs)).Return(mockCmd),
+			mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
+			mockExec.EXPECT().Command(iptablesExecutable,
+				expectedArgs("filter", "-I", "INPUT", tc.inputRouteArgs)).Return(mockCmd),
+			mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
+			mockExec.EXPECT().Command(iptablesExecutable,
+				expectedArgs("nat", "-A", "OUTPUT", outputRouteArgs)).Return(mockCmd),
+			mockCmd.EXPECT().CombinedOutput().Return([]byte{0}, nil),
+		)
+
+		route, err := NewNetfilterRoute(mockExec)
+		require.NoError(t, err, "Error creating netfilter route object")
+
+		err = route.Create()
+		assert.NoError(t, err, "Error creating route")
+	}
 
-	err = route.Create()
-	assert.NoError(t, err, "Error creating route")
 }
 
 func TestCreateSkipLocalTrafficFilter(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	os.Setenv("ECS_SKIP_LOCALHOST_TRAFFIC_FILTER", "true")
 	defer os.Unsetenv("ECS_SKIP_LOCALHOST_TRAFFIC_FILTER")
 
@@ -200,6 +264,7 @@ func TestCreateErrorOnInputChainCommandError(t *testing.T) {
 }
 
 func TestCreateErrorOnOutputChainCommandError(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -230,6 +295,7 @@ func TestCreateErrorOnOutputChainCommandError(t *testing.T) {
 }
 
 func TestRemove(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -261,6 +327,7 @@ func TestRemove(t *testing.T) {
 }
 
 func TestRemoveSkipLocalTrafficFilter(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	os.Setenv("ECS_SKIP_LOCALHOST_TRAFFIC_FILTER", "true")
 	defer os.Unsetenv("ECS_SKIP_LOCALHOST_TRAFFIC_FILTER")
 
@@ -290,6 +357,7 @@ func TestRemoveSkipLocalTrafficFilter(t *testing.T) {
 }
 
 func TestRemoveAllowIntrospectionOffhostAccess(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	os.Setenv(offhostIntrospectionAccessConfigEnv, "true")
 	defer os.Unsetenv(offhostIntrospectionAccessConfigEnv)
 
@@ -322,6 +390,7 @@ func TestRemoveAllowIntrospectionOffhostAccess(t *testing.T) {
 }
 
 func TestRemoveErrorOnPreroutingChainCommandError(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -352,6 +421,7 @@ func TestRemoveErrorOnPreroutingChainCommandError(t *testing.T) {
 }
 
 func TestRemoveErrorOnOutputChainCommandError(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -382,6 +452,7 @@ func TestRemoveErrorOnOutputChainCommandError(t *testing.T) {
 }
 
 func TestRemoveErrorOnInputChainCommandsErrors(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
@@ -447,10 +518,12 @@ func TestGetLocalhostTrafficFilterInputChainArgs(t *testing.T) {
 }
 
 func TestGetBlockIntrospectionOffhostAccessInputChainArgs(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
+	defaultOffhostIntrospectionInterface, _ = getOffhostIntrospectionInterface()
 	assert.Equal(t, []string{
 		"INPUT",
 		"-p", "tcp",
-		"-i", "eth0",
+		"-i", "ens5",
 		"--dport", "51678",
 		"-j", "DROP",
 	}, getBlockIntrospectionOffhostAccessInputChainArgs())
@@ -478,3 +551,49 @@ func TestGetActionName(t *testing.T) {
 func expectedArgs(table, action, chain string, args []string) []string {
 	return append([]string{"-t", table, action, chain}, args...)
 }
+
+func TestScanIPv4RoutesHappyCase(t *testing.T) {
+	iface, err := scanIPv4RoutesForDefaultInterface(strings.NewReader(testIPV4RouteInput))
+	assert.NoError(t, err)
+	assert.Equal(t, offhostIntrospectionInterface, iface)
+}
+
+func TestScanIPv4RoutesNoDefaultRoute(t *testing.T) {
+	iface, err := scanIPv4RoutesForDefaultInterface(strings.NewReader(""))
+	assert.Error(t, err)
+	assert.Equal(t, "", iface)
+}
+
+func TestScanIPv4RoutesNoDefaultRouteExceptLoopback(t *testing.T) {
+	var testInput = `Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT                                                       
+lo	00000000	01201FAC	0003	0	0	0	00000000	0	0	0
+`
+	iface, err := scanIPv4RoutesForDefaultInterface(strings.NewReader(testInput))
+	assert.Error(t, err)
+	assert.Equal(t, "", iface)
+}
+
+func TestGetOffhostIntrospectionInterfaceWithEnvOverride(t *testing.T) {
+	os.Setenv(offhostIntrospectonAccessInterfaceEnv, "test_iface")
+	defer os.Unsetenv(offhostIntrospectonAccessInterfaceEnv)
+
+	iface, err := getOffhostIntrospectionInterface()
+	assert.NoError(t, err)
+	assert.Equal(t, "test_iface", iface)
+}
+
+func TestGetOffhostIntrospectionInterfaceUseDefaultV4(t *testing.T) {
+	defer overrideIPRouteInput(testIPV4RouteInput)()
+
+	iface, err := getOffhostIntrospectionInterface()
+	assert.NoError(t, err)
+	assert.Equal(t, offhostIntrospectionInterface, iface)
+}
+
+func TestGetOffhostIntrospectionInterfaceFailure(t *testing.T) {
+	defer overrideIPRouteInput("")()
+
+	iface, err := getOffhostIntrospectionInterface()
+	assert.Error(t, err)
+	assert.Equal(t, "", iface)
+}
