@@ -15,7 +15,7 @@ check-option-value() {
 }
 
 usage() {
-    echo "$(basename "$0") [--help] --region REGION --activation-code CODE --activation-id ID [--cluster CLUSTER] [--docker-install-source all|docker|distro|none] [--ecs-version VERSION] [--ecs-endpoint ENDPOINT] [--skip-registration] [--no-start]
+    echo "$(basename "$0") [--help] --region REGION --activation-code CODE --activation-id ID [--cluster CLUSTER] [--enable-gpu] [--docker-install-source all|docker|distro|none] [--ecs-version VERSION] [--ecs-endpoint ENDPOINT] [--skip-registration] [--no-start]
 
   --help
         (optional) display this help message.
@@ -27,12 +27,16 @@ usage() {
         (required) activation code from the create activation command. Not required if --skip-registration is specified.
   --cluster string
         (optional) pass the cluster name that ECS agent will connect too. By default its value is 'default'.
+  --enable-gpu
+        (optional) if this flag is provided, GPU support for ECS will be enabled.
   --docker-install-source
         (optional) Source of docker installation. Possible values are 'all, docker, distro, none'. Defaults to 'all'.
   --ecs-version string
         (optional) Version of the ECS agent rpm/deb package to use. If not specified, default to latest.
   --skip-registration
         (optional) if this is enabled, SSM agent install and instance registration with SSM is skipped.
+  --certs-file
+        (optional) TLS certs for execute command feature. Defaults to searching for certs in known possible locations.
   --no-start
         (optional) if this flag is provided, SSM agent, docker and ECS agent will not be started by the script despite being installed."
 }
@@ -49,6 +53,7 @@ ECS_VERSION=""
 DEB_URL=""
 RPM_URL=""
 ECS_ENDPOINT=""
+CERTS_FILE=""
 # Whether to check signature for the downloaded amazon-ecs-init package. true unless --skip-gpg-check
 # specified. --skip-gpg-check is mostly for testing purpose (so that we can test a custom build of ecs init package
 # without having to sign it).
@@ -105,10 +110,19 @@ while :; do
         ECS_ENDPOINT="$2"
         shift 2
         ;;
+    --certs-file)
+        check-option-value "$1" "$2"
+        CERTS_FILE="$2"
+        shift 2
+        ;;
     --skip-registration)
         SKIP_REGISTRATION=true
         shift 1
         ;;
+    --enable-gpu)
+        GPU_ENABLED=true
+        shift 1
+        ;;
     --no-start)
         NO_START=true
         shift 1
@@ -138,13 +152,6 @@ if [ ! -d /run/systemd/system ]; then
     fail
 fi
 
-if [ -f "/sys/fs/cgroup/cgroup.controllers" ]; then
-    echo "Your system is using cgroups v2, which is not supported by ECS."
-    echo "Please change your system to cgroups v1 and reboot. If your system has grubby, we suggest using the following command:"
-    echo '    sudo grubby --update-kernel=ALL --args="systemd.unified_cgroup_hierarchy=0" && sudo shutdown -r now'
-    fail
-fi
-
 SSM_SERVICE_NAME="amazon-ssm-agent"
 SSM_BIN_NAME="amazon-ssm-agent"
 if systemctl is-enabled snap.amazon-ssm-agent.amazon-ssm-agent.service &>/dev/null; then
@@ -200,12 +207,14 @@ S3_URL_SUFFIX=""
 if grep -q "^cn-" <<< "$REGION"; then
     S3_URL_SUFFIX=".cn"
 fi
+S3_URL="https://s3.${REGION}.amazonaws.com${S3_URL_SUFFIX}"
+SSM_S3_BUCKET="amazon-ssm-$REGION"
 
 if [ -z "$RPM_URL" ]; then
-    RPM_URL="https://s3.${REGION}.amazonaws.com${S3_URL_SUFFIX}/${S3_BUCKET}/$RPM_PKG_NAME"
+    RPM_URL="${S3_URL}/${S3_BUCKET}/$RPM_PKG_NAME"
 fi
 if [ -z "$DEB_URL" ]; then
-    DEB_URL="https://s3.${REGION}.amazonaws.com${S3_URL_SUFFIX}/${S3_BUCKET}/$DEB_PKG_NAME"
+    DEB_URL="${S3_URL}/${S3_BUCKET}/$DEB_PKG_NAME"
 fi
 
 # source /etc/os-release to get the VERSION_ID and ID fields
@@ -224,6 +233,8 @@ elif echo "$ID" | grep centos; then
     DISTRO="centos"
 elif echo "$ID" | grep rhel; then
     DISTRO="rhel"
+elif [[ "$ID" == "amzn" && "$VERSION_ID" == "2" ]]; then
+    DISTRO="al2"
 fi
 
 if [ "$DISTRO" == "rhel" ]; then
@@ -254,7 +265,9 @@ if [ -x "$(command -v dnf)" ]; then
     dnf install -y jq
 elif [ -x "$(command -v yum)" ]; then
     PKG_MANAGER="yum"
-    yum install epel-release -y
+    if [ "$DISTRO" != "al2" ]; then
+        yum install epel-release -y
+    fi
     yum install -y jq
 elif [ -x "$(command -v apt)" ]; then
     PKG_MANAGER="apt"
@@ -296,7 +309,6 @@ register-ssm-agent() {
         else
             echo "Skip starting ssm agent because --no-start is specified."
         fi
-        systemctl start "$SSM_SERVICE_NAME"
         echo "SSM agent has been registered."
     else
         echo "SSM agent is already registered. Managed instance ID: $SSM_MANAGED_INSTANCE_ID"
@@ -311,8 +323,8 @@ install-ssm-agent() {
     else
         local dir
         dir="$(mktemp -d)"
-        local SSM_DEB_URL="https://s3.$REGION.amazonaws.com${S3_URL_SUFFIX}/amazon-ssm-$REGION/latest/debian_$ARCH_ALT/amazon-ssm-agent.deb"
-        local SSM_RPM_URL="https://s3.$REGION.amazonaws.com${S3_URL_SUFFIX}/amazon-ssm-$REGION/latest/linux_$ARCH_ALT/amazon-ssm-agent.rpm"
+        local SSM_DEB_URL="${S3_URL}/${SSM_S3_BUCKET}/latest/debian_$ARCH_ALT/amazon-ssm-agent.deb"
+        local SSM_RPM_URL="${S3_URL}/${SSM_S3_BUCKET}/latest/linux_$ARCH_ALT/amazon-ssm-agent.rpm"
         local SSM_DEB_PKG_NAME="ssm-agent.deb"
         local SSM_RPM_PKG_NAME="ssm-agent.rpm"
 
@@ -353,13 +365,6 @@ ssm-agent-signature-verify() {
     fi
 
     curl-helper "$dir/amazon-ssm-agent.gpg" "https://raw.githubusercontent.com/aws/amazon-ecs-init/master/scripts/amazon-ssm-agent.gpg"
-    local fp
-    fp=$(gpg --quiet --with-colons --with-fingerprint "$dir/amazon-ssm-agent.gpg" | awk -F: '$1 == "fpr" {print $10;}')
-    echo "$fp"
-    if [ "$fp" != "8108A07A9EBE248E3F1C63F254F4F56E693ECA21" ]; then
-        echo "amazon-ssm-agent GPG public key fingerprint verification fail. Stop the installation of the amazon-ssm-agent. Please contact AWS Support."
-        fail
-    fi
     gpg --import "$dir/amazon-ssm-agent.gpg"
 
     if gpg --verify "$1" "$2"; then
@@ -518,6 +523,13 @@ install-ecs-agent() {
     if [ -n "$ECS_ENDPOINT" ]; then
         echo "ECS_BACKEND_HOST=$ECS_ENDPOINT" >>/var/lib/ecs/ecs.config
     fi
+    if [ -n "$GPU_ENABLED" ]; then
+        if ! nvidia-smi &>/dev/null; then
+            echo "Failed to detect GPU with nvidia-smi command. Make sure that GPUs are attached and the latest NVIDIA driver is installed and running."
+            fail
+        fi
+        echo "ECS_ENABLE_GPU_SUPPORT=true" >>/var/lib/ecs/ecs.config
+    fi
     systemctl enable ecs
     if ! $NO_START; then
         systemctl start ecs
@@ -534,10 +546,6 @@ ecs-init-signature-verify() {
         echo "WARNING: gpg command not available on this server, not able to verify amazon-ecs-init package signature."
         ok
         return
-    elif ! command -v dirmngr; then
-        echo "WARNING: dirmngr not installed on this server, not able to verify amazon-ecs-init package signature."
-        ok
-        return
     fi
 
     curl-helper "$dir/amazon-ecs-agent.gpg" "https://raw.githubusercontent.com/aws/amazon-ecs-init/master/scripts/amazon-ecs-agent.gpg"
@@ -579,6 +587,76 @@ wait-agent-start() {
     fail
 }
 
+exec-setup() {
+    find-copy-certs-exec
+    download-ssm-binaries-exec
+}
+
+find-copy-certs-exec() {
+    # Reference for locations: https://golang.org/src/crypto/x509/root_linux.go
+    CERTS_FILES_LOCATIONS=("/etc/ssl/certs/ca-certificates.crt"
+      "/etc/pki/tls/certs/ca-bundle.crt"
+      "/etc/ssl/ca-bundle.pem"
+      "/etc/pki/tls/cacert.pem"
+      "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"
+      "/etc/ssl/cert.pem"
+    )
+    CERTS_PATH="/var/lib/ecs/deps/execute-command/certs"
+
+    echo "Copying certs for exec feature"
+
+    # Determine certs file
+    certs=""
+    if [ -n "$CERTS_FILE" ] && [ -f "$CERTS_FILE" ]; then
+        certs=$CERTS_FILE
+    else
+        if [ -n "$CERTS_FILE" ]; then
+            echo "Provided certs file does not exist, looking for certs in known possible locations"
+        fi
+        for f in "${CERTS_FILES_LOCATIONS[@]}"
+        do
+            if  [ -f "$f" ]; then
+                certs=$f
+                break
+            fi
+        done
+    fi
+
+    # Copy certs to exec directory
+    if [ -z "$certs" ]; then
+        echo "Could not find certificates. Please rerun with --certs-file and provide a valid path"
+        fail
+    else
+        echo "Using $certs"
+        openssl verify -x509_strict "$certs"
+        mkdir -p $CERTS_PATH
+        cp "$certs" $CERTS_PATH/tls-ca-bundle.pem
+        chmod 400 $CERTS_PATH/tls-ca-bundle.pem
+    fi
+    ok
+}
+
+download-ssm-binaries-exec() {
+    BINARY_VERSION="3.1.804.0"
+    BINARY_PATH="/var/lib/ecs/deps/execute-command/bin/${BINARY_VERSION}"
+    BINARY_DOWNLOAD_PATH="ssm-binaries"
+
+    # Download SSM binaries from S3
+    echo "Downloading SSM binaries for exec feature"
+
+    mkdir -p $BINARY_DOWNLOAD_PATH
+    curl "${S3_URL}/${SSM_S3_BUCKET}/${BINARY_VERSION}/linux_$ARCH_ALT/amazon-ssm-agent-binaries.tar.gz" -o ${BINARY_DOWNLOAD_PATH}/amazon-ssm-agent.tar.gz
+    tar -xvf ${BINARY_DOWNLOAD_PATH}/amazon-ssm-agent.tar.gz -C ${BINARY_DOWNLOAD_PATH}/
+
+    # Copy binaries to exec directory
+    mkdir -p ${BINARY_PATH}
+    cp ${BINARY_DOWNLOAD_PATH}/amazon-ssm-agent ${BINARY_PATH}/amazon-ssm-agent
+    cp ${BINARY_DOWNLOAD_PATH}/ssm-agent-worker ${BINARY_PATH}/ssm-agent-worker
+    cp ${BINARY_DOWNLOAD_PATH}/ssm-session-worker ${BINARY_PATH}/ssm-session-worker
+    rm -rf ${BINARY_DOWNLOAD_PATH}
+    ok
+}
+
 show-license() {
     echo ""
     echo "##########################"
@@ -595,6 +673,7 @@ if ! $SKIP_REGISTRATION; then
     install-ssm-agent
 fi
 install-docker "$DOCKER_SOURCE"
+exec-setup
 install-ecs-agent
 wait-agent-start
 show-license
